/*------------------------------------------------------------------------
	PH-COMM.C

	The socket calls of the phone book.
------------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>

#include "ph-comm.h"

extern	struct hostent	*gethostbyname();
extern	struct servent	*getservbyname();
extern	struct protoent	*getprotobyname();

static	int	port_addr( int, char *, struct sockaddr_in *);
static	int	service_addr( char *, char *, char *, struct sockaddr_in *);

/*------------------------------------------------------------------------
	int connect_service(	char *service,
				char *hostname )

RETURN VALUES: A file descriptor for use with read/write ... or
	-1	Service or host not known
	-2	Connect failed

Appropriate error messages are printed to stderr.
------------------------------------------------------------------------*/

int	connect_service(char *service,
			char *hostname )
{
	struct sockaddr_in	addr;
	int			sock;

	/*----------------------------------------------------------------
	Find service address
	----------------------------------------------------------------*/
	if (service_addr(service, "tcp", hostname, &addr) < 0)
	{
		fprintf(stderr, "Service %s/%s or host %s unknown!\n",
			service, "tcp", hostname);
		return -1;
	}

	/*----------------------------------------------------------------
	Create Socket
	----------------------------------------------------------------*/
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0) return -2;

	/*----------------------------------------------------------------
	Connect zum Server
	----------------------------------------------------------------*/
	if (connect(sock, (struct sockaddr*)&addr, sizeof(addr)) < 0)
		return -2;
	
	return sock;
}


/*------------------------------------------------------------------------
	int connect_port(	char *hostname,
				int  port)

RETURN VALUES:  A file descriptor for use with read/write ... or
	-1	Host not known
	-2	Connect failed
------------------------------------------------------------------------*/
int	connect_port(	char *hostname,
			int  port)
{
	struct sockaddr_in	addr;
	int			sock;

	/*----------------------------------------------------------------
	Find address
	----------------------------------------------------------------*/
	if (port_addr( port, hostname, &addr) < 0)
	{
		fprintf(stderr, "Host '%s' unknown!\n", hostname);
		return -1;
	}

	/*----------------------------------------------------------------
	Create Socket
	----------------------------------------------------------------*/
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0)
		return -2;

	/*----------------------------------------------------------------
	Connect zum Server
	----------------------------------------------------------------*/
	if (connect(sock, (struct sockaddr*)&addr, sizeof(addr)) < 0)
		return -2;
	
	return sock;
}


/*------------------------------------------------------------------------
	int port_addr(	int	port,
			char *hostname,
			struct sockaddr_in	*addr)

	Creates a struct sockaddr_in from a port/hostname pair.

RETURN VALUES:
	 0	OK
	-1	Host unknown
------------------------------------------------------------------------*/
static	int	port_addr(	int	port,
			char	*hostname,
			struct sockaddr_in	*addr)
{
	struct	hostent	*hp;

	hp = gethostbyname(hostname);
	endhostent();
	if (hp == 0)
		return -1;

  	memset(addr, 0, sizeof(struct sockaddr_in));
/* *********************************************************
   bzero is substituted by memset due to incompatibility
   with SunOs and Solaris systems
	bzero(addr, sizeof(struct sockaddr_in));
** *********************************************************/
	memcpy (&addr->sin_addr,hp->h_addr, hp->h_length);
	addr->sin_family = hp->h_addrtype;
	addr->sin_port =   htons(port);

	return 0;
}

/*------------------------------------------------------------------------
	int service_addr(	char *service,
				char *protokoll,
				char *hostname,
				struct sockaddr_in	*addr)

	Creates a struct sockaddr_in from a service/protocol/hostname spec.

RETURN VALUES:
	 0	OK
	-1	Service/Protocol unknown
------------------------------------------------------------------------*/

static	int	service_addr(	char *service,
			char *protokoll,
			char *hostname,
			struct sockaddr_in	*addr)
{
	struct	hostent	*hp;
	struct	servent	*sp;
	int		proto;


	sp = getservbyname (service,protokoll);
	endservent();
	if (sp == NULL)
		return -1;

	hp = gethostbyname(hostname);
	endhostent();
	if (hp == 0)
		return -1;

        memset(addr, 0, sizeof(struct sockaddr_in));
/* *********************************************************
   bzero is substituted by memset due to incompatibility
   with SunOs and Solaris systems
	bzero(addr, sizeof(struct sockaddr_in));
** *********************************************************/

	memcpy (&addr->sin_addr,hp->h_addr, hp->h_length);
	addr->sin_family = hp->h_addrtype;
	addr->sin_port =   sp->s_port;

	return 0;
}

/*------------------------------------------------------------------------
			CLIENT FOR THE CERN PHONEBOOK

Module:		ph-comm.c
Purpose:	Convenient wrappers around the socket calls.
Author:		Josef Pellizzari / CN
Last changed:	27 Apr 1996

************************** DISCLAIMER FROM THE AUTHOR ********************
This program has been copied together from modules I used somewhere
else. It has been handed over to Bernd Pollermann ("The Phone People")
so that he doesn't have to write it from scratch. It will probably not
compile on all platforms and its maintenance is the responsibility
of the phone people - certainly not mine! So - please complain there!
**************************************************************************

Description:
The socket calls have all kinds of weird parameters and structures. To
facilitate using them this is a simple wrapper around these calls to
keep the details out of the main program.

Changes:

------------------------------------------------------------------------*/
